from dwollav2.error import Error


class Response:
    def __init__(self, res):
        if (res.status_code >= 400):
            raise Error.map(res)

        self.status = res.status_code
        self.headers = res.headers
        self.body = self._get_body(res)

    def _get_body(self, res):
        try:
            return res.json()
        except:
            return res.text



from dwollav2.auth import auth_for
from dwollav2.token import token_for


class Client:
    ENVIRONMENTS = {
      'production': {
        'auth_url':  'https://www.dwolla.com/oauth/v2/authenticate',
        'token_url': 'https://www.dwolla.com/oauth/v2/token',
        'api_url':   'https://api.dwolla.com'
      },
      'sandbox': {
        'auth_url':  'https://uat.dwolla.com/oauth/v2/authenticate',
        'token_url': 'https://uat.dwolla.com/oauth/v2/token',
        'api_url':   'https://api-uat.dwolla.com'
      }
    }

    def __init__(self, **kwargs):
        self.id = self.key = kwargs.get('id') or kwargs['key']
        self.secret = kwargs['secret']
        self.environment = kwargs.get('environment', 'production')
        if self.environment not in self.ENVIRONMENTS:
            raise ValueError('invalid environment')
        self.on_grant = kwargs.get('on_grant')
        self.Auth = auth_for(self)
        self.Token = token_for(self)

    @property
    def auth_url(self):
        return self.ENVIRONMENTS[self.environment]['auth_url']

    @property
    def token_url(self):
        return self.ENVIRONMENTS[self.environment]['token_url']

    @property
    def api_url(self):
        return self.ENVIRONMENTS[self.environment]['api_url']

    def Token(self, opts = None, **kwargs):
        return Token(self, opts, **kwargs)



try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

import requests

from dwollav2.error import Error
from dwollav2.version import version


session = requests.session()
session.headers.update({'user-agent': 'dwolla-v2-python %s' % version})


def _is_error(res):
    try:
        return 'error' in res.json()
    except:
        return True

def _request_token(client, payload):
    res = session.post(client.token_url, data=payload)
    if _is_error(res):
        raise Error.map(res)
    token = client.Token(res.json())
    if client.on_grant is not None:
        client.on_grant(token)
    return token

def auth_for(_client):
    class Auth:
        def __init__(self, **kwargs):
            self.redirect_uri = kwargs.get('redirect_uri')
            self.scope = kwargs.get('scope')
            self.state = kwargs.get('state')
            self.verified_account = kwargs.get('verified_account')
            self.dwolla_landing = kwargs.get('dwolla_landing')

        @property
        def url(self):
            return '%s?%s' % (_client.auth_url, urlencode(self._query()))

        def callback(self, params):
            if params.get('state') != self.state:
                raise ValueError('invalid state')
            if 'error' in params:
                raise Error.map(params)
            return _request_token(_client, {
                'client_id': _client.id,
                'client_secret': _client.secret,
                'grant_type': 'authorization_code',
                'code': params['code'],
                'redirect_uri': self.redirect_uri
            })

        def _query(self):
            d = {
                'response_type': 'code',
                'client_id': _client.id,
                'redirect_uri': self.redirect_uri,
                'scope': self.scope,
                'state': self.state,
                'verified_account': self.verified_account,
                'dwolla_landing': self.dwolla_landing
            }
            return dict((k, v) for k, v in iter(d.items()) if v is not None)

        @staticmethod
        def client():
            return _request_token(_client, {
                'client_id': _client.id,
                'client_secret': _client.secret,
                'grant_type': 'client_credentials'
            })

        @staticmethod
        def refresh(token):
            return _request_token(_client, {
                'client_id': _client.id,
                'client_secret': _client.secret,
                'grant_type': 'refresh_token',
                'refresh_token': token.refresh_token
            })

    return Auth



import requests

class Error(Exception):
    def __init__(self, res):
        if isinstance(res, requests.Response):
            super(Error, self).__init__(res.text)
            self.status = res.status_code
            self.headers = res.headers
            self.body = self._get_body(res)
        else:
            super(Error, self).__init__(str(res))
            self.body = res

    @staticmethod
    def map(res):
        try:
            if isinstance(res, requests.Response):
                body = res.json()
            else:
                body = res
            c = body.get('code', body.get('error'))
        except:
            c = None

        return {
            'access_denied':             AccessDeniedError,
            'InvalidCredentials':        InvalidCredentialsError,
            'NotFound':                  NotFoundError,
            'BadRequest':                BadRequestError,
            'invalid_grant':             InvalidGrantError,
            'RequestTimeout':            RequestTimeoutError,
            'ExpiredAccessToken':        ExpiredAccessTokenError,
            'invalid_request':           InvalidRequestError,
            'ServerError':               ServerError,
            'Forbidden':                 ForbiddenError,
            'InvalidResourceState':      InvalidResourceStateError,
            'temporarily_unavailable':   TemporarilyUnavailableError,
            'InvalidAccessToken':        InvalidAccessTokenError,
            'InvalidScope':              InvalidScopeError,
            'unauthorized_client':       UnauthorizedClientError,
            'InvalidAccountStatus':      InvalidAccountStatusError,
            'unsupported_grant_type':    UnsupportedGrantTypeError,
            'InvalidApplicationStatus':  InvalidApplicationStatusError,
            'InvalidVersion':            InvalidVersionError,
            'unsupported_response_type': UnsupportedResponseTypeError,
            'invalid_client':            InvalidClientError,
            'method_not_allowed':        MethodNotAllowedError,
            'ValidationError':           ValidationError,
            'TooManyRequests':           TooManyRequestsError,
            'Conflict':                  ConflictError
        }.get(c, Error)(res)

    def _get_body(self, res):
        try:
            return res.json()
        except:
            return res.text

class AccessDeniedError(Error):
    pass

class InvalidCredentialsError(Error):
    pass

class NotFoundError(Error):
    pass

class BadRequestError(Error):
    pass

class InvalidGrantError(Error):
    pass

class RequestTimeoutError(Error):
    pass

class ExpiredAccessTokenError(Error):
    pass

class InvalidRequestError(Error):
    pass

class ServerError(Error):
    pass

class ForbiddenError(Error):
    pass

class InvalidResourceStateError(Error):
    pass

class TemporarilyUnavailableError(Error):
    pass

class InvalidAccessTokenError(Error):
    pass

class InvalidScopeError(Error):
    pass

class UnauthorizedClientError(Error):
    pass

class InvalidAccountStatusError(Error):
    pass

class UnsupportedGrantTypeError(Error):
    pass

class InvalidApplicationStatusError(Error):
    pass

class InvalidVersionError(Error):
    pass

class UnsupportedResponseTypeError(Error):
    pass

class InvalidClientError(Error):
    pass

class MethodNotAllowedError(Error):
    pass

class ValidationError(Error):
    pass

class TooManyRequestsError(Error):
    pass

class ConflictError(Error):
    pass



import os
import sys
import warnings

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='dwollav2',
    version='1.2.0',
    packages=['dwollav2'],
    install_requires=[
        'requests>=2.9.1',
        'future>=0.15.2'
    ],
    test_suite='dwollav2.test.all',
    url='https://docsv2.dwolla.com',
    license='MIT',
    author='Stephen Ausman',
    author_email='stephen@dwolla.com',
    long_description=open('README.md').read(),
    description='Official Dwolla V2 API client',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)
